<?php
/* Copyright (C) XEHub <https://www.xehub.io> */
/**
 * @file  index.php
 * @author XEHub (developers@xpressengine.com)
 * @brief Start page
 *
 * @mainpage XpressEngine
 * @section intro introduction
 *
 * "XpressEngine (XE)" is an opensource and being developed in the opensource project.\n
 * For more information, please see the link below.
 * - Official website: http://www.xpressengine.com
 * - Offcial Repository: https://github.com/xpressengine/xe-core
 * \n
 * "XpressEngine (XE)" is free software; you can redistribute it and/or \n
 * modify it under the terms of the GNU Lesser General Public \n
 * License as published by the Free Software Foundation; either \n
 * version 2.1 of the License, or (at your option) any later version. \n
 * \n
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * \n
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 **/

/**
 * @brief Declare constants for generic use and for checking to avoid a direct call from the Web
 **/
define('__XE__',   TRUE);
/**
 * @brief Include the necessary configuration files
 **/
require dirname(__FILE__) . '/config/config.inc.php';

/*
	For server side logging and debugging in chrome console (Chrome extension 'Chrome Logger')
	https://chrome.google.com/webstore/detail/chrome-logger/noaneddfkdjfnfdakjjmocngnfkfehhd
	https://ddart.net/xe/xe
*/
define ('_SAVE_COMPILE_TARGET_', 1);
include_once "ChromePhp.php";
//ChromePhp::setLogFile("./ChromePhp.log");

/**
 * @brief Initialize by creating Context object
 * Set all Request Argument/Environment variables
 **/
$oContext = Context::getInstance();
$oContext->init();

/**
 * @brief If default_url is set and it is different from the current url, attempt to redirect for SSO authentication and then process the module
 **/
if($oContext->checkSSO())
{
	$oModuleHandler = new ModuleHandler();

	try
	{
		if($oModuleHandler->init())
		{
			$oModuleHandler->displayContent($oModuleHandler->procModule());
		}
	}
	catch(Exception $e)
	{
		htmlHeader();
		echo Context::getLang($e->getMessage());
		htmlFooter();
	}
	catch(Error $e)
	{
		htmlHeader();
		$msg = Array("<table cellpadding=5><col nowrap align=right width=150><col>");
		$msg[] = "<tr><td align=right>Name    </td><td>" . get_class($e)    . "</td></tr>";
		$msg[] = "<tr><td align=right>Message </td><td>" . $e->getMessage() . "</td></tr>";
		$msg[] = "<tr><td align=right>File    </td><td>" . $e->getFile()    . "</td></tr>";
		$msg[] = "<tr><td align=right>Line    </td><td>" . $e->getLine()    . "</td></tr>";
		$msg[] = "<tr><td align=right>Code    </td><td>" . $e->getCode()    . "</td></tr>";
		$msg[] = "</table>";
		echo implode(PHP_EOL, $msg);
		htmlFooter();
		ChromePhp::log($e);
	}
}

$oContext->close();

/* End of file index.php */
/* Location: ./index.php */
